// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/aiplatform/v1/index_endpoint_service.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_AIPLATFORM_V1_INDEX_ENDPOINT_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_AIPLATFORM_V1_INDEX_ENDPOINT_CLIENT_H

#include "google/cloud/aiplatform/v1/index_endpoint_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace aiplatform_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// A service for managing Vertex AI's IndexEndpoints.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class IndexEndpointServiceClient {
 public:
  explicit IndexEndpointServiceClient(
      std::shared_ptr<IndexEndpointServiceConnection> connection,
      Options opts = {});
  ~IndexEndpointServiceClient();

  ///@{
  /// @name Copy and move support
  IndexEndpointServiceClient(IndexEndpointServiceClient const&) = default;
  IndexEndpointServiceClient& operator=(IndexEndpointServiceClient const&) =
      default;
  IndexEndpointServiceClient(IndexEndpointServiceClient&&) = default;
  IndexEndpointServiceClient& operator=(IndexEndpointServiceClient&&) = default;
  ///@}

  ///@{
  /// @name Equality
  friend bool operator==(IndexEndpointServiceClient const& a,
                         IndexEndpointServiceClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(IndexEndpointServiceClient const& a,
                         IndexEndpointServiceClient const& b) {
    return !(a == b);
  }
  ///@}

  // clang-format off
  ///
  /// Creates an IndexEndpoint.
  ///
  /// @param parent  Required. The resource name of the Location to create the IndexEndpoint in.
  ///  Format: `projects/{project}/locations/{location}`
  /// @param index_endpoint  Required. The IndexEndpoint to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.IndexEndpoint] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.CreateIndexEndpointRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L143}
  /// [google.cloud.aiplatform.v1.IndexEndpoint]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint.proto#L36}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::IndexEndpoint>>
  CreateIndexEndpoint(
      std::string const& parent,
      google::cloud::aiplatform::v1::IndexEndpoint const& index_endpoint,
      Options opts = {});

  // clang-format off
  ///
  /// Creates an IndexEndpoint.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.aiplatform.v1.CreateIndexEndpointRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.IndexEndpoint] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.CreateIndexEndpointRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L143}
  /// [google.cloud.aiplatform.v1.IndexEndpoint]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint.proto#L36}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::IndexEndpoint>>
  CreateIndexEndpoint(
      google::cloud::aiplatform::v1::CreateIndexEndpointRequest const& request,
      Options opts = {});

  // clang-format off
  ///
  /// Gets an IndexEndpoint.
  ///
  /// @param name  Required. The name of the IndexEndpoint resource.
  ///  Format:
  ///  `projects/{project}/locations/{location}/indexEndpoints/{index_endpoint}`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return the result of the RPC. The response message type
  ///     ([google.cloud.aiplatform.v1.IndexEndpoint])
  ///     is mapped to a C++ class using the [Protobuf mapping rules].
  ///     If the request fails, the [`StatusOr`] contains the error details.
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.GetIndexEndpointRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L166}
  /// [google.cloud.aiplatform.v1.IndexEndpoint]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint.proto#L36}
  ///
  // clang-format on
  StatusOr<google::cloud::aiplatform::v1::IndexEndpoint> GetIndexEndpoint(
      std::string const& name, Options opts = {});

  // clang-format off
  ///
  /// Gets an IndexEndpoint.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.aiplatform.v1.GetIndexEndpointRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return the result of the RPC. The response message type
  ///     ([google.cloud.aiplatform.v1.IndexEndpoint])
  ///     is mapped to a C++ class using the [Protobuf mapping rules].
  ///     If the request fails, the [`StatusOr`] contains the error details.
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.GetIndexEndpointRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L166}
  /// [google.cloud.aiplatform.v1.IndexEndpoint]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint.proto#L36}
  ///
  // clang-format on
  StatusOr<google::cloud::aiplatform::v1::IndexEndpoint> GetIndexEndpoint(
      google::cloud::aiplatform::v1::GetIndexEndpointRequest const& request,
      Options opts = {});

  // clang-format off
  ///
  /// Lists IndexEndpoints in a Location.
  ///
  /// @param parent  Required. The resource name of the Location from which to list the
  ///  IndexEndpoints. Format: `projects/{project}/locations/{location}`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return a [StreamRange](@ref google::cloud::StreamRange)
  ///     to iterate of the results. See the documentation of this type for
  ///     details. In brief, this class has `begin()` and `end()` member
  ///     functions returning a iterator class meeting the
  ///     [input iterator requirements]. The value type for this iterator is a
  ///     [`StatusOr`] as the iteration may fail even after some values are
  ///     retrieved successfully, for example, if there is a network disconnect.
  ///     An empty set of results does not indicate an error, it indicates
  ///     that there are no resources meeting the request criteria.
  ///     On a successful iteration the `StatusOr<T>` contains elements of type
  ///     [google.cloud.aiplatform.v1.IndexEndpoint], or rather,
  ///     the C++ class generated by Protobuf from that type. Please consult the
  ///     Protobuf documentation for details on the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.IndexEndpoint]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint.proto#L36}
  /// [google.cloud.aiplatform.v1.ListIndexEndpointsRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L180}
  ///
  // clang-format on
  StreamRange<google::cloud::aiplatform::v1::IndexEndpoint> ListIndexEndpoints(
      std::string const& parent, Options opts = {});

  // clang-format off
  ///
  /// Lists IndexEndpoints in a Location.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.aiplatform.v1.ListIndexEndpointsRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return a [StreamRange](@ref google::cloud::StreamRange)
  ///     to iterate of the results. See the documentation of this type for
  ///     details. In brief, this class has `begin()` and `end()` member
  ///     functions returning a iterator class meeting the
  ///     [input iterator requirements]. The value type for this iterator is a
  ///     [`StatusOr`] as the iteration may fail even after some values are
  ///     retrieved successfully, for example, if there is a network disconnect.
  ///     An empty set of results does not indicate an error, it indicates
  ///     that there are no resources meeting the request criteria.
  ///     On a successful iteration the `StatusOr<T>` contains elements of type
  ///     [google.cloud.aiplatform.v1.IndexEndpoint], or rather,
  ///     the C++ class generated by Protobuf from that type. Please consult the
  ///     Protobuf documentation for details on the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.IndexEndpoint]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint.proto#L36}
  /// [google.cloud.aiplatform.v1.ListIndexEndpointsRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L180}
  ///
  // clang-format on
  StreamRange<google::cloud::aiplatform::v1::IndexEndpoint> ListIndexEndpoints(
      google::cloud::aiplatform::v1::ListIndexEndpointsRequest request,
      Options opts = {});

  // clang-format off
  ///
  /// Updates an IndexEndpoint.
  ///
  /// @param index_endpoint  Required. The IndexEndpoint which replaces the resource on the server.
  /// @param update_mask  Required. The update mask applies to the resource. See
  ///  [google.protobuf.FieldMask][google.protobuf.FieldMask].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return the result of the RPC. The response message type
  ///     ([google.cloud.aiplatform.v1.IndexEndpoint])
  ///     is mapped to a C++ class using the [Protobuf mapping rules].
  ///     If the request fails, the [`StatusOr`] contains the error details.
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.IndexEndpoint]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint.proto#L36}
  /// [google.cloud.aiplatform.v1.UpdateIndexEndpointRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L241}
  ///
  // clang-format on
  StatusOr<google::cloud::aiplatform::v1::IndexEndpoint> UpdateIndexEndpoint(
      google::cloud::aiplatform::v1::IndexEndpoint const& index_endpoint,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  // clang-format off
  ///
  /// Updates an IndexEndpoint.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.aiplatform.v1.UpdateIndexEndpointRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return the result of the RPC. The response message type
  ///     ([google.cloud.aiplatform.v1.IndexEndpoint])
  ///     is mapped to a C++ class using the [Protobuf mapping rules].
  ///     If the request fails, the [`StatusOr`] contains the error details.
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.IndexEndpoint]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint.proto#L36}
  /// [google.cloud.aiplatform.v1.UpdateIndexEndpointRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L241}
  ///
  // clang-format on
  StatusOr<google::cloud::aiplatform::v1::IndexEndpoint> UpdateIndexEndpoint(
      google::cloud::aiplatform::v1::UpdateIndexEndpointRequest const& request,
      Options opts = {});

  // clang-format off
  ///
  /// Deletes an IndexEndpoint.
  ///
  /// @param name  Required. The name of the IndexEndpoint resource to be deleted.
  ///  Format:
  ///  `projects/{project}/locations/{location}/indexEndpoints/{index_endpoint}`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.DeleteOperationMetadata] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.DeleteIndexEndpointRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L253}
  /// [google.cloud.aiplatform.v1.DeleteOperationMetadata]: @googleapis_reference_link{google/cloud/aiplatform/v1/operation.proto#L52}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::DeleteOperationMetadata>>
  DeleteIndexEndpoint(std::string const& name, Options opts = {});

  // clang-format off
  ///
  /// Deletes an IndexEndpoint.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.aiplatform.v1.DeleteIndexEndpointRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.DeleteOperationMetadata] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.DeleteIndexEndpointRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L253}
  /// [google.cloud.aiplatform.v1.DeleteOperationMetadata]: @googleapis_reference_link{google/cloud/aiplatform/v1/operation.proto#L52}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::DeleteOperationMetadata>>
  DeleteIndexEndpoint(
      google::cloud::aiplatform::v1::DeleteIndexEndpointRequest const& request,
      Options opts = {});

  // clang-format off
  ///
  /// Deploys an Index into this IndexEndpoint, creating a DeployedIndex within
  /// it.
  /// Only non-empty Indexes can be deployed.
  ///
  /// @param index_endpoint  Required. The name of the IndexEndpoint resource into which to deploy an
  ///  Index. Format:
  ///  `projects/{project}/locations/{location}/indexEndpoints/{index_endpoint}`
  /// @param deployed_index  Required. The DeployedIndex to be created within the IndexEndpoint.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.DeployIndexResponse] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.DeployIndexRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L267}
  /// [google.cloud.aiplatform.v1.DeployIndexResponse]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L284}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::DeployIndexResponse>>
  DeployIndex(
      std::string const& index_endpoint,
      google::cloud::aiplatform::v1::DeployedIndex const& deployed_index,
      Options opts = {});

  // clang-format off
  ///
  /// Deploys an Index into this IndexEndpoint, creating a DeployedIndex within
  /// it.
  /// Only non-empty Indexes can be deployed.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.aiplatform.v1.DeployIndexRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.DeployIndexResponse] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.DeployIndexRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L267}
  /// [google.cloud.aiplatform.v1.DeployIndexResponse]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L284}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::DeployIndexResponse>>
  DeployIndex(google::cloud::aiplatform::v1::DeployIndexRequest const& request,
              Options opts = {});

  // clang-format off
  ///
  /// Undeploys an Index from an IndexEndpoint, removing a DeployedIndex from it,
  /// and freeing all resources it's using.
  ///
  /// @param index_endpoint  Required. The name of the IndexEndpoint resource from which to undeploy an
  ///  Index. Format:
  ///  `projects/{project}/locations/{location}/indexEndpoints/{index_endpoint}`
  /// @param deployed_index_id  Required. The ID of the DeployedIndex to be undeployed from the
  ///  IndexEndpoint.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.UndeployIndexResponse] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.UndeployIndexRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L301}
  /// [google.cloud.aiplatform.v1.UndeployIndexResponse]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L319}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::UndeployIndexResponse>>
  UndeployIndex(std::string const& index_endpoint,
                std::string const& deployed_index_id, Options opts = {});

  // clang-format off
  ///
  /// Undeploys an Index from an IndexEndpoint, removing a DeployedIndex from it,
  /// and freeing all resources it's using.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.aiplatform.v1.UndeployIndexRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.UndeployIndexResponse] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.UndeployIndexRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L301}
  /// [google.cloud.aiplatform.v1.UndeployIndexResponse]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L319}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::UndeployIndexResponse>>
  UndeployIndex(
      google::cloud::aiplatform::v1::UndeployIndexRequest const& request,
      Options opts = {});

  // clang-format off
  ///
  /// Update an existing DeployedIndex under an IndexEndpoint.
  ///
  /// @param index_endpoint  Required. The name of the IndexEndpoint resource into which to deploy an
  ///  Index. Format:
  ///  `projects/{project}/locations/{location}/indexEndpoints/{index_endpoint}`
  /// @param deployed_index  Required. The DeployedIndex to be updated within the IndexEndpoint.
  ///  Currently, the updatable fields are [DeployedIndex][automatic_resources]
  ///  and [DeployedIndex][dedicated_resources]
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.MutateDeployedIndexResponse] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.MutateDeployedIndexRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L330}
  /// [google.cloud.aiplatform.v1.MutateDeployedIndexResponse]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L349}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::MutateDeployedIndexResponse>>
  MutateDeployedIndex(
      std::string const& index_endpoint,
      google::cloud::aiplatform::v1::DeployedIndex const& deployed_index,
      Options opts = {});

  // clang-format off
  ///
  /// Update an existing DeployedIndex under an IndexEndpoint.
  ///
  /// @param request Unary RPCs, such as the one wrapped by this
  ///     function, receive a single `request` proto message which includes all
  ///     the inputs for the RPC. In this case, the proto message is a
  ///     [google.cloud.aiplatform.v1.MutateDeployedIndexRequest].
  ///     Proto messages are converted to C++ classes by Protobuf, using the
  ///     [Protobuf mapping rules].
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return A [`future`] that becomes satisfied when the LRO
  ///     ([Long Running Operation]) completes or the polling policy in effect
  ///     for this call is exhausted. The future is satisfied with an error if
  ///     the LRO completes with an error or the polling policy is exhausted.
  ///     In this case the [`StatusOr`] returned by the future contains the
  ///     error. If the LRO completes successfully the value of the future
  ///     contains the LRO's result. For this RPC the result is a
  ///     [google.cloud.aiplatform.v1.MutateDeployedIndexResponse] proto message.
  ///     The C++ class representing this message is created by Protobuf, using
  ///     the [Protobuf mapping rules].
  ///
  /// [Protobuf mapping rules]: https://protobuf.dev/reference/cpp/cpp-generated/
  /// [input iterator requirements]: https://en.cppreference.com/w/cpp/named_req/InputIterator
  /// [Long Running Operation]: https://google.aip.dev/151
  /// [`std::string`]: https://en.cppreference.com/w/cpp/string/basic_string
  /// [`future`]: @ref google::cloud::future
  /// [`StatusOr`]: @ref google::cloud::StatusOr
  /// [`Status`]: @ref google::cloud::Status
  /// [google.cloud.aiplatform.v1.MutateDeployedIndexRequest]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L330}
  /// [google.cloud.aiplatform.v1.MutateDeployedIndexResponse]: @googleapis_reference_link{google/cloud/aiplatform/v1/index_endpoint_service.proto#L349}
  ///
  // clang-format on
  future<StatusOr<google::cloud::aiplatform::v1::MutateDeployedIndexResponse>>
  MutateDeployedIndex(
      google::cloud::aiplatform::v1::MutateDeployedIndexRequest const& request,
      Options opts = {});

 private:
  std::shared_ptr<IndexEndpointServiceConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace aiplatform_v1
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_AIPLATFORM_V1_INDEX_ENDPOINT_CLIENT_H
